﻿//
// Copyright (c) 2011, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;


namespace MindFusion.Diagramming.Wpf.Samples.CS.MultiViews
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow()
		{
			InitializeComponent();
			myDiagram = new Diagram();
			viewList = new List<DiagramView>();

			AllowDrop = true;
			DragOver += new DragEventHandler(MainWindow_DragOver);
			Drop += new DragEventHandler(MainWindow_Drop);
		}

		void MainWindow_Drop(object sender, DragEventArgs e)
		{
			var gridLayer = e.OriginalSource as UIElement;
			if (gridLayer != null && VisualTreeHelper.GetParent(gridLayer) is Diagram)
			{
				var diagram = (Diagram)VisualTreeHelper.GetParent(gridLayer);
				e.Effects = DragDropEffects.Copy;
				e.Handled = true;

				var point = e.GetPosition(diagram.DocumentPlane);
				diagram.Factory.CreateShapeNode(point, new Size(50, 50))
					.Text = (string)e.Data.GetData(typeof(string));
			}
		}

		void MainWindow_DragOver(object sender, DragEventArgs e)
		{
			e.Effects = DragDropEffects.Copy;
			e.Handled = true;
		}

		private void myAdd_Click(object sender, RoutedEventArgs e)
		{
			var view = new DiagramView();
			//view.ZoomFactor = 300;

			var window = new UI.Wpf.Window();
			window.Width = 200;
			window.Height = 200;
			window.Content = view;
			window.Show(windowHost);

			view.Focusable = true;
			view.Diagram = diagram;
			view.PreviewMouseWheel += OnPreviewMouseWheel;
		}

		void Diagram_Drop(object sender, DragEventArgs e)
		{
			var diagram = (Diagram)sender;

			e.Effects = DragDropEffects.Copy;
			e.Handled = true;

			var point = e.GetPosition(diagram.DocumentPlane);
			diagram.Factory.CreateShapeNode(point, new Size(50, 50))
				.Text = (string)e.Data.GetData(typeof(string));
		}

		void Diagram_DragOver(object sender, DragEventArgs e)
		{
			e.Effects = DragDropEffects.Copy;
			e.Handled = true;
		}

		private void myRemove_Click(object sender, RoutedEventArgs e)
		{
			if (viewList.Count > 0)
			{
				windowHost.Windows.Remove(windowHost.Windows[windowHost.Windows.Count - 1]);
				viewList.Remove(viewList[viewList.Count - 1]);
			}
		}


		Diagram myDiagram;
		List<DiagramView> viewList;

		bool firstRun = true;
		Diagram diagram;

		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			var view = new DiagramView();
			//view.ZoomFactor = 300;
			diagram = new Diagram();
			diagram.Factory.CreateShapeNode(30, 30, 60, 60).Selected = true;

			var window = new UI.Wpf.Window();
			window.Width = 200;
			window.Height = 200;
			window.Content = view;
			window.Show(windowHost);

			view.Focusable = true;
			view.Diagram = diagram;
			view.PreviewMouseWheel += OnPreviewMouseWheel;

			Keyboard.Focus(view); // does not work from here

			LayoutUpdated += window_LayoutUpdated;
		}

		void window_LayoutUpdated(object sender, EventArgs e)
		{
			if (firstRun)
			{
				//Keyboard.Focus(diagram);
				firstRun = false;
			}
		}

		private void OnPreviewMouseWheel(object sender, MouseWheelEventArgs e)
		{
			var view = sender as DiagramView;
			if (view != null)
			{
				view.SetZoomFactor(
					view.ZoomFactor + e.Delta / 20.0,
					view.ClientToDoc(e.GetPosition(view)));
				e.Handled = true;
			}
		}
	}
}